/*
 * Generation of include directives.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "includes.h"



/* Write include directives having certain forms. */

void write_general_include(const char *filename, FILE *fp)
{
  fprintf(fp, "#include %s\n", filename);
}

void write_include(const char *name, const char *suffix, FILE *fp)
{
  fprintf(fp, "#include \"%s%s\"\n", name, suffix);
}

/* Write include statements. */

void write_includes(struct include *inc, FILE *fp)
{
  if (inc == NULL)
    return;

  /* The list is reversed. */

  write_includes(inc->tail, fp);
  write_general_include(inc->filename, fp);
}

void write_includes_separator(struct include *inc, FILE *fp)
{
  if (inc != NULL)
    fputs("\n", fp);
}
