/*
 * Provide a calculation server.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <math.h>
#include <stdio.h>

#include <ipc/server.h>

#include "calc_server.h"



/* Component interface. */

class server_Calc : public Calc
{
public:
  long add(int left, int right, int *result)
  {
    *result = left + right;
    return L4_EOK;
  }

  long subtract(int left, int right, int *result)
  {
    *result = left - right;
    return L4_EOK;
  }

  long multiply(int left, int right, int *result)
  {
    *result = left * right;
    return L4_EOK;
  }

  long divide(int numerator, int denominator, int *result)
  {
    *result = numerator / denominator;
    return L4_EOK;
  }

  long pow(double base, double exponent, double *result)
  {
    *result = powf(base, exponent);
    return L4_EOK;
  }
};



int main(void)
{
  /* Calculator encapsulation. */

  server_Calc obj;

  /* Server capability. */

  l4_cap_idx_t server;

  /* Register a server associating it with the given object. */

  long err = ipc_server_bind("server", (l4_umword_t) &obj, &server);

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* Wait for messages, dispatching to the handler. */

  ipc_server_loop_for(Calc, &obj);

  return 0;
}
